%takes parameter values used to generate limiting problems and uses them to
%simulate finite-sample size of conventional test statistics and two-step
%tests.

s = RandStream('mt19937ar','Seed',1);
if heterosked==0;
    draws=unidrnd(instruments,corr_obs,1);
    shocks=randn(corr_obs,2);
    for m=1:instruments
        var_temp=var_draw_store(:,:,m);
        %Reduce correlation of errors
        shocks(draws==m,:)=shocks(draws==m,:)*var_temp^.5;
    end
    homosked_var=cov(shocks);
end

draws=unidrnd(instruments,corr_obs,1);
Z=zeros(obs,instruments);
X=zeros(corr_obs,1);
shocks=randn(corr_obs,2);
for m=1:instruments
    if heterosked==1;
        var_temp=var_draw_store(:,:,m);
    else
        var_temp=homosked_var;
    end
    %Modify covariance of shocks
    var_temp(1,1)=var_temp(1,1)*var_scaling;
    var_temp(1,2)=var_temp(1,2)*corr_scaling*var_scaling^.5;
    var_temp(2,1)=var_temp(2,1)*corr_scaling*var_scaling^.5;
    Z(draws==m,:)=repmat(Z_store(:,m)',sum(draws==m),1);
    shocks(draws==m,:)=shocks(draws==m,:)*var_temp^.5;
end
cov_base=Z.*repmat(shocks(:,2),1,instruments);
cov_mat=cov(cov_base);
clear cov_base
pi_noncentrality=mu_base'*cov_mat^-1*mu_base;

prelimobs=obs/10;
prelimreps=repcount/10;
CD_rej_rate_2SLS=zeros(length(pi_grid_init),1);
CD_rej_rate_liml=zeros(length(pi_grid_init),1);
CD_rej_rate_ROT=zeros(length(pi_grid_init),1);
for k=1:length(pi_grid_init)
    pibase=pi_grid_init(k);
    pi=mu_base*pibase/prelimobs^.5/pi_noncentrality^.5*instruments^.5;
    
    Cragg_Donald_store=zeros(prelimreps,1);
    S_store=zeros(repcount,1);
    clear Z
    
    for n=1:prelimreps;
        s = RandStream('mt19937ar','Seed',n);
        RandStream.setGlobalStream(s);
        
        draws=unidrnd(instruments,prelimobs,1);
        Z=zeros(prelimobs,instruments);
        X=zeros(prelimobs,1);
        shocks=randn(prelimobs,2);
        for m=1:instruments
            if heterosked==1;
                var_temp=var_draw_store(:,:,m);
            else
                var_temp=homosked_var;
            end
            %Modify covariance of shocks
            var_temp(1,1)=var_temp(1,1)*var_scaling;
            var_temp(1,2)=var_temp(1,2)*corr_scaling*var_scaling^.5;
            var_temp(2,1)=var_temp(2,1)*corr_scaling*var_scaling^.5;
            Z(draws==m,:)=repmat(Z_store(:,m)',sum(draws==m),1);
            shocks(draws==m,:)=shocks(draws==m,:)*var_temp^.5;
            X(draws==m,:)=Z(draws==m,:)*pi+shocks(draws==m,2);
        end
        
        pi_hat=(Z'*Z)^-1*Z'*X;
        V_hat=X-Z*pi_hat;
        
        if heterosked==0
            %non-robust Cragg-Donald statistic
            Sigma_VV=(V_hat'*V_hat-sum(V_hat)^2)/(prelimobs-instruments);
            Cragg_Donald_stat=Sigma_VV^-1*(X'*Z)*(Z'*Z)^-1*(Z'*X)/instruments;
        else
            e_base=Z.*repmat(V_hat,1,size(Z,2));
            Sigma_pi=(Z'*Z)*((e_base-repmat(mean(e_base,1),prelimobs,1))'*...
                (e_base-repmat(mean(e_base,1),prelimobs,1)))^-1*(Z'*Z);
            Cragg_Donald_stat=pi_hat'*Sigma_pi*pi_hat/instruments;
        end
        Cragg_Donald_store(n,1)=Cragg_Donald_stat;
    end
    %Calculate Pretests and rejection rates.  Note that CD tests null of weak
    %ID, while mine "tests" null of strong ID.
    if instruments==5
        CD_2SLS=Cragg_Donald_store>26.87;
        CD_liml=Cragg_Donald_store>4.84;
        CD_ROT=Cragg_Donald_store>10;
    elseif instruments==10;    
        CD_2SLS=Cragg_Donald_store>38.54;
        CD_liml=Cragg_Donald_store>3.68;
        CD_ROT=Cragg_Donald_store>10;
    elseif instruments==20
        CD_2SLS=Cragg_Donald_store>62.3;
        CD_liml=Cragg_Donald_store>3.21;
        CD_ROT=Cragg_Donald_store>10;
    end
    %pretest_liml_store=max(Wald_liml_CI_store-CLR_pretest_CI_store,[],2)>0;
    
    CD_rej_rate_2SLS(k,1)=mean(CD_2SLS);
    CD_rej_rate_liml(k,1)=mean(CD_liml);
    CD_rej_rate_ROT(k,1)=mean(CD_ROT);
end
relevant_2SLS=(CD_rej_rate_2SLS<.99)&(CD_rej_rate_2SLS>0.01);
relevant_liml=(CD_rej_rate_liml<.99)&(CD_rej_rate_liml>0.01);
relevant_ROT=(CD_rej_rate_ROT<.99)&(CD_rej_rate_ROT>0.01);
relevant=(relevant_2SLS+relevant_liml+relevant_ROT)>0;

pi_grid_trim=pi_grid_init(relevant);
pi_grid_trim2=pi_grid_trim;
for k=1:length(pi_grid_trim)
    pi_grid_trim2=[pi_grid_trim2 pi_grid_trim(k)-pigrid_window:pigrid_window_step:pi_grid_trim(k)+pigrid_window];
end

pibasegrid=[pi_grid_trim2 pi_grid_rough];
pibasegrid=sort(unique(pibasegrid));
pibasegrid(pibasegrid<0)=[];


beta1=0;
CD_rej_rate_2SLS=zeros(length(pibasegrid),1);
CD_rej_rate_liml=zeros(length(pibasegrid),1);
CD_rej_rate_ROT=zeros(length(pibasegrid),1);
CD_mean=zeros(length(pibasegrid),1);
S_size=zeros(length(pibasegrid),1);
Wald_2SLS_size=zeros(length(pibasegrid),1);
Wald_2SGMM_size=zeros(length(pibasegrid),1);
Wald_LIML_size=zeros(length(pibasegrid),1);
Wald_CUE_size=zeros(length(pibasegrid),1);
CD_2SLS_size=zeros(length(pibasegrid),1);
CD_2SGMM_size=zeros(length(pibasegrid),1);
CD_LIML_size=zeros(length(pibasegrid),1);
CD_CUE_size=zeros(length(pibasegrid),1);
ROT_2SLS_size=zeros(length(pibasegrid),1);
ROT_2SGMM_size=zeros(length(pibasegrid),1);
ROT_LIML_size=zeros(length(pibasegrid),1);
ROT_CUE_size=zeros(length(pibasegrid),1);
for k=1:length(pibasegrid)
    pibase=pibasegrid(k);
    pi=mu_base*pibase/obs^.5/pi_noncentrality^.5*instruments^.5;
    
    Wald_2SLS_store=zeros(repcount,1);
    Wald_2SGMM_store=zeros(repcount,1);
    Wald_LIML_store=zeros(repcount,1);
    Wald_CUE_store=zeros(repcount,1);
    Cragg_Donald_store=zeros(repcount,1);
    S_store=zeros(repcount,1);
    clear Z
    k/length(pibasegrid)
    
    for n=1:repcount;
        s = RandStream('mt19937ar','Seed',n);
        RandStream.setGlobalStream(s);
        
        draws=unidrnd(instruments,obs,1);
        Z=zeros(obs,instruments);
        X=zeros(obs,1);
        shocks=randn(obs,2);
        Y=zeros(obs,1);
        for m=1:instruments
            if heterosked==1;
                var_temp=var_draw_store(:,:,m);
            else
                var_temp=homosked_var;
            end
            %Modify covariance of shocks
            var_temp(1,1)=var_temp(1,1)*var_scaling;
            var_temp(1,2)=var_temp(1,2)*corr_scaling*var_scaling^.5;
            var_temp(2,1)=var_temp(2,1)*corr_scaling*var_scaling^.5;
            Z(draws==m,:)=repmat(Z_store(:,m)',sum(draws==m),1);
            shocks(draws==m,:)=shocks(draws==m,:)*var_temp^.5;
            X(draws==m,:)=Z(draws==m,:)*pi+shocks(draws==m,2);
            Y(draws==m,:)=Z(draws==m,:)*pi*beta1+shocks(draws==m,1);
        end
        

        gmat=@(b) repmat(Y-X*b,1,instruments).*Z;
        g=@(b) sum(gmat(b))';
        qmat=repmat(X,1,instruments).*Z;
        
        omega_base=[repmat(Y,1,instruments).*Z repmat(X,1,instruments).*Z];
        Omega=cov(omega_base);
        Omega11=Omega(1:instruments,1:instruments);
        Omega12=Omega(1:instruments,instruments+1:2*instruments);
        Omega21=Omega(instruments+1:2*instruments,1:instruments);
        Omega22=Omega(instruments+1:2*instruments,instruments+1:2*instruments);
        Sigma_gg=@(b) obs*(Omega11-b*(Omega12+Omega21)+b^2*Omega22);
        
        %Sigma_gg=@(b) (gmat(b)-repmat(mean(gmat(b),1),obs,1))'*(gmat(b)-repmat(mean(gmat(b),1),obs,1));
        
        
        
        G=1/obs*Z'*X;
        W_homosked=(1/obs*(Z'*Z))^-1;
        
        %Calculate 2sls and corresponding Wald Statistic
        X_hat=Z*(Z'*Z)^-1*(Z'*X);
        b_2sls=(X_hat'*X_hat)^-1*X_hat'*Y;
        Omega_2sls=1/obs*Sigma_gg(b_2sls);
        Sigma_2sls=(G'*W_homosked*G)^-1*G'*W_homosked*Omega_2sls*W_homosked*G*(G'*W_homosked*G)^-1;
        W_2sls=obs*(b_2sls-beta1)'*Sigma_2sls^-1*(b_2sls-beta1);
        
        %Calculate 2SGMM and corresponding Wald Statistic
        b_1sgmm=(X'*Z)*(Z'*Y)/((X'*Z)*(Z'*X));
        W=Sigma_gg(b_1sgmm)^-1;
        b_2sgmm=(X'*Z)*W*(Z'*Y)/((X'*Z)*W*(Z'*X));
        Omega_2sgmm=1/obs*Sigma_gg(b_2sgmm);
        Sigma_2sgmm=(G'*Omega_2sgmm^-1*G)^-1;
        W_2sgmm=obs*(b_2sgmm-beta1)'*Sigma_2sgmm^-1*(b_2sgmm-beta1);
        
        %Calculate LIML and corresponding Wald Statistic
        ZX=Z'*X;
        ZY=Z'*Y;
        YY=Y'*Y;
        YX=Y'*X;
        XX=X'*X;
        %liml_obj=@(b) (ZY-b*ZX)'*W_homosked*(ZY-b*ZX)/sum((Y-X*b).^2);
        liml_obj=@(b) (ZY-b*ZX)'*W_homosked*(ZY-b*ZX)/(YY-2*b*YX+b^2*XX);
        b_liml=fminsearch(liml_obj,b_2sls,options);
        Omega_liml=1/obs*Sigma_gg(b_liml);
        Sigma_liml=(G'*W_homosked*G)^-1*G'*W_homosked*Omega_liml*W_homosked*G*(G'*W_homosked*G)^-1;
        W_liml=obs*(b_liml-beta1)'*Sigma_liml^-1*(b_liml-beta1);
        
        %Calculate CUE and corresponding Wald Statistic
        S=@(b) (ZY-b*ZX)'*Sigma_gg(b)^-1*(ZY-b*ZX);
        b_CUE=fminsearch(S,b_2sgmm,options);
        Omega_CUE=1/obs*Sigma_gg(b_CUE);
        Sigma_CUE=(G'*Omega_CUE^-1*G)^-1;
        W_cue=obs*(b_CUE-beta1)'*Sigma_CUE^-1*(b_CUE-beta1);
        
        pi_hat=(Z'*Z)^-1*Z'*X;
        V_hat=X-Z*pi_hat;
        
        if heterosked==0
            %non-robust Cragg-Donald statistic
            Sigma_VV=(V_hat'*V_hat-sum(V_hat)^2)/(obs-instruments);
            Cragg_Donald_stat=Sigma_VV^-1*(X'*Z)*(Z'*Z)^-1*(Z'*X)/instruments;
        else
            e_base=Z.*repmat(V_hat,1,size(Z,2));
            Sigma_pi=(Z'*Z)*((e_base-repmat(mean(e_base,1),obs,1))'*...
                (e_base-repmat(mean(e_base,1),obs,1)))^-1*(Z'*Z);
            Cragg_Donald_stat=pi_hat'*Sigma_pi*pi_hat/instruments;
        end
        
%         w_2SLS_store(n,1)=W_2sls;
%         w_2SGMM_store(n,1)=W_2sgmm;
%         w_LIML_store(n,1)=W_liml;
%         w_CUE_store(n,1)=W_cue;
        
        Wald_2SLS_store(n,1)=(W_2sls>chi2inv(.95,1));
        Wald_2SGMM_store(n,1)=(W_2sgmm>chi2inv(.95,1));
        Wald_LIML_store(n,1)=(W_liml>chi2inv(.95,1));
        Wald_CUE_store(n,1)=(W_cue>chi2inv(.95,1));
        Cragg_Donald_store(n,1)=Cragg_Donald_stat;
        S_store(n,1)=(g(beta1)'*Sigma_gg(beta1)^-1*g(beta1)>chi2inv(.95,instruments));
    end
    %Calculate Pretests and rejection rates.  Note that CD tests null of weak
    %ID, while mine "tests" null of strong ID.
    if instruments==5
        CD_2SLS=Cragg_Donald_store>26.87;
        CD_liml=Cragg_Donald_store>4.84;
        CD_ROT=Cragg_Donald_store>10;
    elseif instruments==10;    
        CD_2SLS=Cragg_Donald_store>38.54;
        CD_liml=Cragg_Donald_store>3.68;
        CD_ROT=Cragg_Donald_store>10;
    elseif instruments==20
        CD_2SLS=Cragg_Donald_store>62.3;
        CD_liml=Cragg_Donald_store>3.21;
        CD_ROT=Cragg_Donald_store>10;
    end
    %pretest_liml_store=max(Wald_liml_CI_store-CLR_pretest_CI_store,[],2)>0;
    
    CD_rej_rate_2SLS(k,1)=mean(CD_2SLS);
    CD_rej_rate_liml(k,1)=mean(CD_liml);
    CD_rej_rate_ROT(k,1)=mean(CD_ROT);
    CD_mean(k,1)=mean(Cragg_Donald_store);
    
    %Calculate tests based on two-step procedures
    CD_Hybrid_Wald_2SLS_store=Wald_2SLS_store;
    CD_Hybrid_Wald_2SLS_store(CD_2SLS==0,:)=S_store(CD_2SLS==0,:);
    
    CD_Hybrid_Wald_2SGMM_store=Wald_2SGMM_store;
    CD_Hybrid_Wald_2SGMM_store(CD_2SLS==0,:)=S_store(CD_2SLS==0,:);
    
    CD_Hybrid_Wald_LIML_store=Wald_LIML_store;
    CD_Hybrid_Wald_LIML_store(CD_liml==0,:)=S_store(CD_liml==0,:);
    
    CD_Hybrid_Wald_CUE_store=Wald_CUE_store;
    CD_Hybrid_Wald_CUE_store(CD_liml==0,:)=S_store(CD_liml==0,:);
    
    ROT_Hybrid_Wald_2SLS_store=Wald_2SLS_store;
    ROT_Hybrid_Wald_2SLS_store(CD_ROT==0,:)=S_store(CD_ROT==0,:);
    
    ROT_Hybrid_Wald_2SGMM_store=Wald_2SGMM_store;
    ROT_Hybrid_Wald_2SGMM_store(CD_ROT==0,:)=S_store(CD_ROT==0,:);
    
    ROT_Hybrid_Wald_LIML_store=Wald_LIML_store;
    ROT_Hybrid_Wald_LIML_store(CD_ROT==0,:)=S_store(CD_ROT==0,:);
    
    ROT_Hybrid_Wald_CUE_store=Wald_CUE_store;
    ROT_Hybrid_Wald_CUE_store(CD_ROT==0,:)=S_store(CD_ROT==0,:);
    
    
    %Calculate rejection rates
    S_size(k,1)=mean(S_store);
    Wald_2SLS_size(k,1)=mean(Wald_2SLS_store);
    Wald_2SGMM_size(k,1)=mean(Wald_2SGMM_store);
    Wald_LIML_size(k,1)=mean(Wald_LIML_store);
    Wald_CUE_size(k,1)=mean(Wald_CUE_store);
    CD_2SLS_size(k,1)=mean(CD_Hybrid_Wald_2SLS_store);
    CD_2SGMM_size(k,1)=mean(CD_Hybrid_Wald_2SGMM_store);
    CD_LIML_size(k,1)=mean(CD_Hybrid_Wald_LIML_store);
    CD_CUE_size(k,1)=mean(CD_Hybrid_Wald_CUE_store);
    ROT_2SLS_size(k,1)=mean(ROT_Hybrid_Wald_2SLS_store);
    ROT_2SGMM_size(k,1)=mean(ROT_Hybrid_Wald_2SGMM_store);
    ROT_LIML_size(k,1)=mean(ROT_Hybrid_Wald_LIML_store);
    ROT_CUE_size(k,1)=mean(ROT_Hybrid_Wald_CUE_store);
    save sim_backup
end

close all
figure(1)
plot(CD_mean,[Wald_2SLS_size  Wald_2SGMM_size Wald_LIML_size Wald_CUE_size]);
legend('2SLS','2SGMM','LIML','CUE','Location','NE')

figure(2)
plot(CD_mean,[S_size...
    ROT_2SLS_size ROT_2SGMM_size ROT_LIML_size ROT_CUE_size]);
legend('S','CD-2SLS','CD-2SGMM','CD-LIML','CD-CUE','Location','NE')

figure(3)
plot(CD_mean,[S_size...
    CD_2SLS_size CD_2SGMM_size CD_LIML_size CD_CUE_size]);
legend('S','CD-2SLS','CD-2SGMM','CD-LIML','CD-CUE','Location','NE')
